import discord

from ptvp35 import *
from v6d2ctx.context import *
from v6d3music.core.queueaudio import *
from v6d3music.core.ytaservicing import *
from v6d3music.utils.assert_admin import *

__all__ = ('MainAudio',)


class MainAudio(discord.PCMVolumeTransformer):
    def __init__(self, db: DbConnection, queue: QueueAudio):
        self.db = db
        self.queue = queue
        super().__init__(self.queue, volume=self.get())

    async def set(self, volume: float, member: discord.Member):
        assert_admin(member)
        if volume < 0.01:
            raise Explicit('volume too small')
        if volume > 1:
            raise Explicit('volume too big')
        self.volume = volume
        await self.db.set(member.guild.id, volume)

    def get(self) -> float:
        return self.db.get(self.queue.guild.id, 0.2)

    @classmethod
    async def create(cls, servicing: YTAServicing, db: DbConnection, queues: DbConnection, guild: discord.Guild) -> 'MainAudio':
        return cls(db, await QueueAudio.create(servicing, queues, guild))
