import asyncio
from typing import Any, Callable, Coroutine, TypeVar

import aiohttp

T = TypeVar("T")


async def repeat(repeated: Callable[[], Coroutine[Any, Any, T]]) -> T:
    for _ in range(60):
        try:
            return await repeated()
        except aiohttp.ClientConnectorError:
            await asyncio.sleep(1)
    raise RuntimeError("cannot reach sessionservice")


async def set_config(key: str, value: Any) -> None:
    json = {"key": key, "value": value}
    async def call() -> None:
        async with aiohttp.ClientSession() as s, s.post("http://sessionservice/config/", json=json) as response:
            if response.status != 200:
                raise RuntimeError("config request failed")
    await repeat(call)
